@@ -7,6 +7,10 @@ import time
 import sys
 import leidenalg
 from tqdm import tqdm
+import uuid
+import os, re
+import subprocess
+from joblib import Parallel, delayed
 
 
 class ClusterResults(object):
@@ -76,10 +80,10 @@ class PhenographCluster(AbstractAffinityMatClusterer):
 
 class LeidenCluster(AbstractAffinityMatClusterer):
 
-    def __init__(self, contin_runs=10, n_leiden_iterations=-1,
+    def __init__(self, numseedstotry=10, n_leiden_iterations=-1,
                  partitiontype=leidenalg.ModularityVertexPartition,
                  affmat_transformer=None, verbose=True): 
-        self.contin_runs = contin_runs
+        self.numseedstotry = numseedstotry
         self.n_leiden_iterations = n_leiden_iterations
         self.partitiontype = partitiontype
         self.affmat_transformer = affmat_transformer
@@ -99,7 +103,6 @@ class LeidenCluster(AbstractAffinityMatClusterer):
             assert np.min(orig_affinity_mat) >= 0,\
                     np.min(orig_affinity_mat)
 
-
         if (self.verbose):
             print("Beginning preprocessing + Leiden")
             sys.stdout.flush()
@@ -114,9 +117,9 @@ class LeidenCluster(AbstractAffinityMatClusterer):
         best_quality = None
 
         if (self.verbose):
-            toiterover = tqdm(range(self.contin_runs))
+            toiterover = tqdm(range(self.numseedstotry))
         else:
-            toiterover = range(self.contin_runs)
+            toiterover = range(self.numseedstotry)
 
         #if an initclustering is specified, we would want to try the Leiden
         # both with and without that initialization and take the one that
@@ -145,65 +148,128 @@ class LeidenCluster(AbstractAffinityMatClusterer):
                               quality=best_quality)
 
 
-#class LeidenCluster(AbstractAffinityMatClusterer):
-#
-#    def __init__(self, contin_runs=10, n_leiden_iterations=-1,
-#                 partitiontype=leidenalg.ModularityVertexPartition,
-#                 affmat_transformer=None, verbose=True): 
-#        self.contin_runs = contin_runs
-#        self.n_leiden_iterations = n_leiden_iterations
-#        self.partitiontype = partitiontype
-#        self.affmat_transformer = affmat_transformer
-#        self.verbose = verbose
-#
-#    def __call__(self, orig_affinity_mat, initclusters):
-#        #replace nan values with zeros
-#        orig_affinity_mat = np.nan_to_num(orig_affinity_mat)
-#        assert np.min(orig_affinity_mat) >= 0, np.min(orig_affinity_mat)
-#
-#        if (self.verbose):
-#            print("Beginning preprocessing + Leiden")
-#            sys.stdout.flush()
-#        all_start = time.time()
-#        if (self.affmat_transformer is not None):
-#            affinity_mat = self.affmat_transformer(orig_affinity_mat)
-#        else:
-#            affinity_mat = orig_affinity_mat
-#
-#        the_graph = get_igraph_from_adjacency(adjacency=affinity_mat)
-#        best_clustering = None
-#        best_quality = None
-#
-#        if (self.verbose):
-#            toiterover = tqdm(range(self.contin_runs))
-#        else:
-#            toiterover = range(self.contin_runs)
-#
-#        #if an initclustering is specified, we would want to try the Leiden
-#        # both with and without that initialization and take the one that
-#        # gets the best modularity
-#        initclusters_to_try_list = [None]
-#        if (initclusters is not None):
-#            initclusters_to_try_list.append(initclusters)
-#
-#        for seed in toiterover:
-#            for initclusters_to_try in initclusters_to_try_list:
-#                partition = leidenalg.find_partition(
-#                    the_graph, self.partitiontype,
-#                    weights=(np.array(the_graph.es['weight'])
-#                             .astype(np.float64)),
-#                    n_iterations=self.n_leiden_iterations,
-#                    initial_membership=initclusters_to_try,
-#                    seed=seed*100)
-#                quality = partition.quality()
-#                if ((best_quality is None) or (quality > best_quality)):
-#                    best_quality = quality
-#                    best_clustering = np.array(partition.membership)
-#                    if (self.verbose):
-#                        print("Quality:",best_quality)
-#                        sys.stdout.flush()
-#        return ClusterResults(cluster_indices=best_clustering,
-#                              quality=best_quality)
+def run_leiden(fileprefix, use_initclusters, n_vertices,
+               partitiontype, n_leiden_iterations, seed):
+
+    lpath = os.path.join(os.path.dirname(__file__), "run_leiden")
+
+    args = [lpath,
+            "--sources_idxs_file", fileprefix+"_sources.npy",
+            "--targets_idxs_file", fileprefix+"_targets.npy",
+            "--weights_file", fileprefix+"_weights.npy",
+            "--n_vertices", str(n_vertices),
+            "--partition_type", partitiontype.__name__, 
+            "--n_iterations", str(n_leiden_iterations),
+            "--seed", str(seed)]
+    if (use_initclusters):
+        args = args + ["--initial_membership_file",
+                       fileprefix+"_initclusters.npy"]
+
+    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    out, err = p.communicate()
+
+    parse_membership = False
+    membership = []
+    for i, line in enumerate(out.decode().splitlines()):
+        if (parse_membership):
+            membership.append(int(line))
+        if line.startswith("Quality:"):
+            quality = float(line.split(" ")[1])
+        if line.startswith("Membership:"):
+            parse_membership = True 
+
+    return quality, np.array(membership)
+
+
+class LeidenClusterParallel(AbstractAffinityMatClusterer):
+
+    def __init__(self, n_jobs,
+                 numseedstotry=10, n_leiden_iterations=-1,
+                 partitiontype=leidenalg.ModularityVertexPartition,
+                 affmat_transformer=None, verbose=True): 
+        self.numseedstotry = numseedstotry 
+        self.n_leiden_iterations = n_leiden_iterations
+        self.partitiontype = partitiontype
+        self.n_jobs = n_jobs
+        self.affmat_transformer = affmat_transformer
+        self.verbose = verbose
+
+    def __call__(self, orig_affinity_mat, initclusters):
+
+        #assert there are no nan values in data
+        #assert that the min affinity is >= 0
+        if scipy.sparse.issparse(orig_affinity_mat):
+            assert np.sum(np.isnan(orig_affinity_mat.data))==0
+            #assert that the min affinity is >= 0
+            assert np.min(orig_affinity_mat.data) >= 0,\
+                    np.min(orig_affinity_mat.data)
+        else:
+            assert np.sum(np.isnan(orig_affinity_mat))==0
+            assert np.min(orig_affinity_mat) >= 0,\
+                    np.min(orig_affinity_mat)
+
+        if (self.verbose):
+            print("Beginning preprocessing + Leiden")
+            sys.stdout.flush()
+        all_start = time.time()
+        if (self.affmat_transformer is not None):
+            affinity_mat = self.affmat_transformer(orig_affinity_mat)
+        else:
+            affinity_mat = orig_affinity_mat
+
+        the_graph = get_igraph_from_adjacency(adjacency=affinity_mat)
+        best_clustering = None
+        best_quality = None
+
+        toiterover = range(self.numseedstotry)
+
+        #if an initclustering is specified, we would want to try the Leiden
+        # both with and without that initialization and take the one that
+        # gets the best modularity
+        initclusters_to_try_list = [False]
+        if (initclusters is not None):
+            initclusters_to_try_list.append(True)
+
+
+        #write out the contents of affinity_mat and initclusters if applicable
+        uid = uuid.uuid1().hex
+        
+        sources, targets = affinity_mat.nonzero()
+        weights = affinity_mat[sources, targets]
+
+        np.save(uid+"_sources.npy", sources)
+        np.save(uid+"_targets.npy", targets)
+        np.save(uid+"_weights.npy", weights.A1) #A1 is the same as ravel()
+
+        if (initclusters is not None):
+            np.save(uid+"_initclusters.npy", initclusters)
+
+        for use_initclusters in initclusters_to_try_list:
+
+            parallel_leiden_results = (
+                Parallel(n_jobs=self.n_jobs,
+                         verbose=self.verbose)(
+                 delayed(run_leiden)(uid, use_initclusters,
+                                     affinity_mat.shape[0],
+                                     self.partitiontype,
+                                     self.n_leiden_iterations, seed*100)
+                 for seed in toiterover)) 
+
+            for quality,membership in parallel_leiden_results:
+                if ((best_quality is None) or (quality > best_quality)):
+                    best_quality = quality
+                    best_clustering = membership
+                    if (self.verbose):
+                        print("Quality:",best_quality)
+                        sys.stdout.flush()
+
+        # clean up
+        for f in os.listdir(os.getcwd()):
+            if re.search(uid, f):
+                os.remove(f)
+
+        return ClusterResults(cluster_indices=best_clustering,
+                              quality=best_quality)
 
 
 #From: https://github.com/theislab/scanpy/blob/8131b05b7a8729eae3d3a5e146292f377dd736f7/scanpy/_utils.py#L159
@@ -217,10 +283,7 @@ def get_igraph_from_adjacency(adjacency, directed=None):
     g = ig.Graph(directed=directed)
     g.add_vertices(adjacency.shape[0])  # this adds adjacency.shap[0] vertices
     g.add_edges(list(zip(sources, targets)))
-    try:
-        g.es['weight'] = weights
-    except:
-        pass
+    g.es['weight'] = weights
     if g.vcount() != adjacency.shape[0]:
         print('WARNING: The constructed graph has only '
               +str(g.vcount())+' nodes. '
